@@ -308,6 +308,10 @@ static NSDictionary<NSString*,NSNumber*> *global_shortcuts = nil;
     return NSTerminateCancel;
 }
 
+- (BOOL)applicationSupportsSecureRestorableState:(NSApplication *)app {
+    return YES;
+}
+
 static GLFWapplicationshouldhandlereopenfun handle_reopen_callback = NULL;
 
 - (BOOL)applicationShouldHandleReopen:(NSApplication *)sender hasVisibleWindows:(BOOL)flag
@@ -354,8 +358,7 @@ static GLFWapplicationwillfinishlaunchingfun finish_launching_callback = NULL;
         else
             createMenuBar();
     }
-    if (finish_launching_callback)
-        finish_launching_callback();
+    if (finish_launching_callback) finish_launching_callback(false);
 }
 
 - (BOOL)application:(NSApplication *)sender openFile:(NSString *)filename {
@@ -406,6 +409,7 @@ static GLFWapplicationwillfinishlaunchingfun finish_launching_callback = NULL;
 
 - (void)applicationDidFinishLaunching:(NSNotification *)notification
 {
+    if (finish_launching_callback) finish_launching_callback(true);
     (void)notification;
     [NSApp stop:nil];
 
@@ -575,7 +579,7 @@ is_shiftable_shortcut(int scv) {
     return scv == kSHKMoveFocusToActiveOrNextWindow || scv == kSHKMoveFocusToNextWindow;
 }
 
-#define USEFUL_MODS(x) (x & (NSEventModifierFlagShift | NSEventModifierFlagOption | NSEventModifierFlagCommand | NSEventModifierFlagControl))
+#define USEFUL_MODS(x) (x & (NSEventModifierFlagShift | NSEventModifierFlagOption | NSEventModifierFlagCommand | NSEventModifierFlagControl | NSEventModifierFlagFunction))
 
 static void
 build_global_shortcuts_lookup(void) {
@@ -787,10 +791,11 @@ GLFWAPI GLFWapplicationwillfinishlaunchingfun glfwSetApplicationWillFinishLaunch
     return previous;
 }
 
-int _glfwPlatformInit(void)
+int _glfwPlatformInit(bool *supports_window_occlusion)
 {
     @autoreleasepool {
 
+    *supports_window_occlusion = true;
     _glfw.ns.helper = [[GLFWHelper alloc] init];
 
     [NSThread detachNewThreadSelector:@selector(doNothing:)
@@ -1016,12 +1021,14 @@ static NSLock *tick_lock = NULL;
 
 void _glfwDispatchTickCallback(void) {
     if (tick_lock && tick_callback) {
-        [tick_lock lock];
-        while(tick_callback_requested) {
-            tick_callback_requested = false;
-            tick_callback(tick_callback_data);
+        while(true) {
+            bool do_call = false;
+            [tick_lock lock];
+            if (tick_callback_requested) { do_call = true; tick_callback_requested = false; }
+            [tick_lock unlock];
+            if (do_call) tick_callback(tick_callback_data);
+            else break;
         }
-        [tick_lock unlock];
     }
 }
 
@@ -1135,3 +1142,5 @@ void _glfwPlatformUpdateTimer(unsigned long long timer_id, monotonic_t interval,
         }
     }
 }
+
+void _glfwPlatformInputColorScheme(GLFWColorScheme appearance UNUSED) { }
