@@ -30,10 +30,8 @@
 #include <Carbon/Carbon.h>
 #if defined(__OBJC__)
 #import <Cocoa/Cocoa.h>
-#import <CoreVideo/CoreVideo.h>
 #else
 typedef void* id;
-typedef void* CVDisplayLinkRef;
 #endif
 
 // NOTE: Many Cocoa enum values have been renamed and we need to build across
@@ -69,7 +67,7 @@ typedef void* CVDisplayLinkRef;
 typedef int (* GLFWcocoatextinputfilterfun)(int,int,unsigned int, unsigned long);
 typedef bool (* GLFWapplicationshouldhandlereopenfun)(int);
 typedef bool (* GLFWhandleurlopen)(const char*);
-typedef void (* GLFWapplicationwillfinishlaunchingfun)(void);
+typedef void (* GLFWapplicationwillfinishlaunchingfun)(bool);
 typedef bool (* GLFWcocoatogglefullscreenfun)(GLFWwindow*);
 typedef void (* GLFWcocoarenderframefun)(GLFWwindow*);
 
@@ -127,6 +125,7 @@ typedef struct _GLFWwindowNS
     id              delegate;
     id              view;
     id              layer;
+    pid_t           previous_front_most_application;
 
     bool            maximized;
     bool            retina;
@@ -152,20 +151,21 @@ typedef struct _GLFWwindowNS
     GLFWcocoatogglefullscreenfun toggleFullscreenCallback;
     // Dead key state
     UInt32 deadKeyState;
+
+    // Layer shell windows
+    struct {
+        bool is_active;
+        GLFWLayerShellConfig config;
+    } layer_shell;
+
     // Whether a render frame has been requested for this window
     bool renderFrameRequested;
     GLFWcocoarenderframefun renderFrameCallback;
     // update cursor after switching desktops with Mission Control
     bool delayed_cursor_update_requested;
+    GLFWcocoarenderframefun resizeCallback;
 } _GLFWwindowNS;
 
-typedef struct _GLFWDisplayLinkNS
-{
-    CVDisplayLinkRef displayLink;
-    CGDirectDisplayID displayID;
-    monotonic_t lastRenderFrameRequestedAt, first_unserviced_render_frame_request_at;
-} _GLFWDisplayLinkNS;
-
 // Cocoa-specific global data
 //
 typedef struct _GLFWlibraryNS
@@ -183,7 +183,7 @@ typedef struct _GLFWlibraryNS
     id                  nibObjects;
 
     char                keyName[64];
-    char                text[256];
+    char                text[512];
     CGPoint             cascadePoint;
     // Where to place the cursor when re-enabled
     double              restoreCursorPosX, restoreCursorPosY;
@@ -198,10 +198,6 @@ typedef struct _GLFWlibraryNS
         CFStringRef     kPropertyUnicodeKeyLayoutData;
     } tis;
 
-    struct {
-        _GLFWDisplayLinkNS entries[256];
-        size_t count;
-    } displayLinks;
     // the callback to handle url open events
     GLFWhandleurlopen url_open_callback;
 
@@ -243,12 +239,16 @@ float _glfwTransformYNS(float y);
 
 void* _glfwLoadLocalVulkanLoaderNS(void);
 
+
+// display links
 void _glfwClearDisplayLinks(void);
 void _glfwRestartDisplayLinks(void);
-void _glfwDispatchTickCallback(void);
+unsigned _glfwCreateDisplayLink(CGDirectDisplayID);
 void _glfwDispatchRenderFrame(CGDirectDisplayID);
-void _glfwShutdownCVDisplayLink(unsigned long long, void*);
+void _glfwRequestRenderFrame(_GLFWwindow *w);
+
+// event loop
+void _glfwDispatchTickCallback(void);
 void _glfwCocoaPostEmptyEvent(void);
-void _glfw_create_cv_display_link(_GLFWDisplayLinkNS *entry);
-_GLFWDisplayLinkNS* _glfw_create_display_link(CGDirectDisplayID);
+
 uint32_t vk_to_unicode_key_with_current_layout(uint16_t keycode);
