@@ -7,18 +7,17 @@ import (
 	"encoding/base64"
 	"fmt"
 	"image"
+	"io"
 	"os"
 	"path/filepath"
+	"slices"
 	"strings"
 	"sync"
 
-	"kitty/tools/tty"
-	"kitty/tools/tui/loop"
-	"kitty/tools/utils"
-	"kitty/tools/utils/images"
-
-	"golang.org/x/exp/maps"
-	"golang.org/x/exp/slices"
+	"github.com/kovidgoyal/kitty/tools/tty"
+	"github.com/kovidgoyal/kitty/tools/tui/loop"
+	"github.com/kovidgoyal/kitty/tools/utils"
+	"github.com/kovidgoyal/kitty/tools/utils/images"
 )
 
 var _ = fmt.Print
@@ -110,7 +109,7 @@ func (self *Output) commit() {
 		return
 	}
 	if self.image_needs_conversion {
-		self.dest.Seek(0, os.SEEK_SET)
+		self.dest.Seek(0, io.SeekStart)
 		img, _, err := image.Decode(self.dest)
 		self.dest.Close()
 		os.Remove(self.dest.Name())
@@ -202,9 +201,13 @@ func unescape_metadata_value(k, x string) (ans string) {
 	return x
 }
 
-func encode_bytes(metadata map[string]string, payload []byte) string {
+func Encode_bytes(metadata map[string]string, payload []byte) string {
 	ans := strings.Builder{}
-	ans.Grow(2048)
+	enc_payload := ""
+	if len(payload) > 0 {
+		enc_payload = base64.StdEncoding.EncodeToString(payload)
+	}
+	ans.Grow(2048 + len(enc_payload))
 	ans.WriteString("\x1b]")
 	ans.WriteString(OSC_NUMBER)
 	ans.WriteString(";")
@@ -218,14 +221,14 @@ func encode_bytes(metadata map[string]string, payload []byte) string {
 	}
 	if len(payload) > 0 {
 		ans.WriteString(";")
-		ans.WriteString(base64.StdEncoding.EncodeToString(payload))
+		ans.WriteString(enc_payload)
 	}
 	ans.WriteString("\x1b\\")
 	return ans.String()
 }
 
 func encode(metadata map[string]string, payload string) string {
-	return encode_bytes(metadata, utils.UnsafeStringToBytes(payload))
+	return Encode_bytes(metadata, utils.UnsafeStringToBytes(payload))
 }
 
 func error_from_status(status string) error {
@@ -283,7 +286,7 @@ func parse_aliases(raw []string) (map[string][]string, error) {
 }
 
 func run_get_loop(opts *Options, args []string) (err error) {
-	lp, err := loop.New(loop.NoAlternateScreen, loop.NoRestoreColors, loop.NoMouseTracking)
+	lp, err := loop.New(loop.NoAlternateScreen, loop.NoRestoreColors, loop.NoMouseTracking, loop.NoInBandResizeNotifications)
 	if err != nil {
 		return err
 	}
@@ -364,7 +367,7 @@ func run_get_loop(opts *Options, args []string) (err error) {
 					}
 				}
 				if len(requested_mimes) > 0 {
-					lp.QueueWriteString(encode(basic_metadata, strings.Join(maps.Keys(requested_mimes), " ")))
+					lp.QueueWriteString(encode(basic_metadata, strings.Join(utils.Keys(requested_mimes), " ")))
 				} else {
 					lp.Quit(0)
 				}
