@@ -152,6 +152,8 @@ mask_kitty_signals_process_wide(PyObject *self UNUSED, PyObject *a UNUSED) {
     Py_RETURN_NONE;
 }
 
+static int verify_peer_uid = false;
+
 static PyObject *
 new_childmonitor_object(PyTypeObject *type, PyObject *args, PyObject UNUSED *kwds) {
     ChildMonitor *self;
@@ -160,7 +162,7 @@ new_childmonitor_object(PyTypeObject *type, PyObject *args, PyObject UNUSED *kwd
     int ret;
 
     if (the_monitor) { PyErr_SetString(PyExc_RuntimeError, "Can have only a single ChildMonitor instance"); return NULL; }
-    if (!PyArg_ParseTuple(args, "OO|ii", &death_notify, &dump_callback, &talk_fd, &listen_fd)) return NULL;
+    if (!PyArg_ParseTuple(args, "OO|iip", &death_notify, &dump_callback, &talk_fd, &listen_fd, &verify_peer_uid)) return NULL;
     if ((ret = pthread_mutex_init(&children_lock, NULL)) != 0) {
         PyErr_Format(PyExc_RuntimeError, "Failed to create children_lock mutex: %s", strerror(ret));
         return NULL;
@@ -532,7 +534,7 @@ parse_input(ChildMonitor *self) {
         DECREF_CHILD(scratch[i]);
     }
     if (reload_config_called) {
-        call_boss(load_config_file, "");
+        call_boss(load_config_file, NULL);
     }
     return input_read;
 }
@@ -652,7 +654,7 @@ pyset_iutf8(ChildMonitor *self, PyObject *args) {
 static bool
 cursor_needs_render(Window *w) {
 #define cri w->render_data.screen->cursor_render_info
-    return w->cursor_visible_at_last_render != cri.is_visible || w->render_data.screen->last_rendered.cursor_x != cri.x || w->render_data.screen->last_rendered.cursor_y != cri.y || w->last_cursor_shape != cri.shape;
+    return w->cursor_opacity_at_last_render != cri.opacity || w->render_data.screen->last_rendered.cursor_x != cri.x || w->render_data.screen->last_rendered.cursor_y != cri.y || w->last_cursor_shape != cri.shape;
 #undef cri
 }
 
@@ -669,22 +671,24 @@ collect_cursor_info(CursorRenderInfo *ans, Window *w, monotonic_t now, OSWindow
         cursor = rd->screen->paused_rendering.expires_at ? &rd->screen->paused_rendering.cursor : rd->screen->cursor;
         ans->x = cursor->x; ans->y = cursor->y;
     }
-    ans->is_visible = false;
+    ans->opacity = 0;
     if (rd->screen->scrolled_by || !screen_is_cursor_visible(rd->screen)) return cursor_needs_render(w);
     monotonic_t time_since_start_blink = now - os_window->cursor_blink_zero_time;
     bool cursor_blinking = OPT(cursor_blink_interval) > 0 && !cursor->non_blinking && os_window->is_focused && (OPT(cursor_stop_blinking_after) == 0 || time_since_start_blink <= OPT(cursor_stop_blinking_after));
-    bool do_draw_cursor = true;
+    ans->opacity = 1;
     if (cursor_blinking) {
-        int t = monotonic_t_to_ms(time_since_start_blink);
-        int d = monotonic_t_to_ms(OPT(cursor_blink_interval));
-        int n = t / d;
-        do_draw_cursor = n % 2 == 0 ? true : false;
-        monotonic_t bucket = ms_to_monotonic_t((monotonic_t)(n + 1) * d);
-        monotonic_t delay = bucket - time_since_start_blink;
-        set_maximum_wait(delay);
+        if (animation_is_valid(OPT(animation.cursor))) {
+            monotonic_t duration = OPT(cursor_blink_interval) * 2;
+            monotonic_t time_into_cycle = time_since_start_blink % duration;
+            double frac_into_cycle = (double)time_into_cycle / (double)duration;
+            ans->opacity = (float)apply_easing_curve(OPT(animation.cursor), frac_into_cycle, duration);
+            set_maximum_wait(ANIMATION_SAMPLE_WAIT);
+        } else {
+            monotonic_t n = time_since_start_blink / OPT(cursor_blink_interval);
+            ans->opacity = 1 - n % 2;
+            set_maximum_wait((n + 1) * OPT(cursor_blink_interval) - time_since_start_blink);
+        }
     }
-    if (!do_draw_cursor) { ans->is_visible = false; return cursor_needs_render(w); }
-    ans->is_visible = true;
     ans->shape = cursor->shape ? cursor->shape : OPT(cursor_shape);
     ans->is_focused = os_window->is_focused;
     return cursor_needs_render(w);
@@ -713,9 +717,9 @@ prepare_to_render_os_window(OSWindow *os_window, monotonic_t now, unsigned int *
         }
         if (send_cell_data_to_gpu(TD.vao_idx, TD.xstart, TD.ystart, TD.dx, TD.dy, TD.screen, os_window)) needs_render = true;
     }
-    if (OPT(mouse_hide_wait) > 0 && !is_mouse_hidden(os_window)) {
-        if (now - os_window->last_mouse_activity_at >= OPT(mouse_hide_wait)) hide_mouse(os_window);
-        else set_maximum_wait(OPT(mouse_hide_wait) - now + os_window->last_mouse_activity_at);
+    if (OPT(mouse_hide.hide_wait) > 0 && !is_mouse_hidden(os_window)) {
+        if (now - os_window->last_mouse_activity_at >= OPT(mouse_hide.hide_wait)) hide_mouse(os_window);
+        else set_maximum_wait(OPT(mouse_hide.hide_wait) - now + os_window->last_mouse_activity_at);
     }
     Tab *tab = os_window->tabs + os_window->active_tab;
     *active_window_bg = OPT(background);
@@ -747,12 +751,26 @@ prepare_to_render_os_window(OSWindow *os_window, monotonic_t now, unsigned int *
                 WD.screen->cursor_render_info.is_focused = os_window->is_focused;
                 set_os_window_title_from_window(w, os_window);
                 *active_window_bg = window_bg;
+                if (OPT(cursor_trail)) {
+                    if (update_cursor_trail(&tab->cursor_trail, w, now, os_window)) {
+                        needs_render = true;
+                        // A max wait of zero causes key input processing to be
+                        // slow so handle the case of OPT(repaint_delay) == 0, see https://github.com/kovidgoyal/kitty/pull/8066
+                        set_maximum_wait(MAX(OPT(repaint_delay), ms_to_monotonic_t(1ll)));
+                    } else if (OPT(cursor_trail) > now - WD.screen->cursor->position_changed_by_client_at) {
+                        // If update_cursor_trail failed due to time threshold, the trail animation
+                        // should be evaluated again shortly. Schedule next update when enough time
+                        // has passed since the cursor was last moved.
+                        set_maximum_wait(OPT(cursor_trail) - now + WD.screen->cursor->position_changed_by_client_at);
+                    }
+                }
+
             } else {
                 if (WD.screen->cursor_render_info.render_even_when_unfocused) {
                     if (collect_cursor_info(&WD.screen->cursor_render_info, w, now, os_window)) needs_render = true;
                     WD.screen->cursor_render_info.is_focused = false;
                 } else {
-                    WD.screen->cursor_render_info.is_visible = false;
+                    WD.screen->cursor_render_info.opacity = 0;
                 }
             }
             if (scan_for_animated_images) {
@@ -775,7 +793,7 @@ draw_resizing_text(OSWindow *w) {
     if (monotonic() - w->created_at > ms_to_monotonic_t(1000) && w->live_resize.num_of_resize_events > 1) {
         char text[32] = {0};
         unsigned int width = w->live_resize.width, height = w->live_resize.height;
-        snprintf(text, sizeof(text), "%u x %u cells", width / w->fonts_data->cell_width, height / w->fonts_data->cell_height);
+        snprintf(text, sizeof(text), "%u x %u cells", width / w->fonts_data->fcm.cell_width, height / w->fonts_data->fcm.cell_height);
         StringCanvas rendered = render_simple_text(w->fonts_data, text);
         if (rendered.canvas) {
             draw_centered_alpha_mask(w, width, height, rendered.width, rendered.height, rendered.canvas, OPT(background_opacity));
@@ -794,23 +812,24 @@ render_prepared_os_window(OSWindow *os_window, unsigned int active_window_id, co
     br->is_dirty = false;
     if (TD.screen && os_window->num_tabs >= OPT(tab_bar_min_tabs)) draw_cells(TD.vao_idx, &TD, os_window, true, true, false, NULL);
     unsigned int num_of_visible_windows = 0;
+    Window *active_window = NULL;
     for (unsigned int i = 0; i < tab->num_windows; i++) { if (tab->windows[i].visible) num_of_visible_windows++; }
     for (unsigned int i = 0; i < tab->num_windows; i++) {
         Window *w = tab->windows + i;
         if (w->visible && WD.screen) {
             bool is_active_window = i == tab->active_window;
+            if (is_active_window) active_window = w;
             draw_cells(WD.vao_idx, &WD, os_window, is_active_window, false, num_of_visible_windows == 1, w);
-            if (WD.screen->start_visual_bell_at != 0) {
-                set_maximum_wait(OPT(repaint_delay));
-            }
-            w->cursor_visible_at_last_render = WD.screen->cursor_render_info.is_visible; w->last_cursor_shape = WD.screen->cursor_render_info.shape;
+            if (WD.screen->start_visual_bell_at != 0) set_maximum_wait(ANIMATION_SAMPLE_WAIT);
+            w->cursor_opacity_at_last_render = WD.screen->cursor_render_info.opacity; w->last_cursor_shape = WD.screen->cursor_render_info.shape;
         }
     }
+    if (OPT(cursor_trail) && tab->cursor_trail.needs_render) draw_cursor_trail(&tab->cursor_trail, active_window);
     if (os_window->live_resize.in_progress) draw_resizing_text(os_window);
     swap_window_buffers(os_window);
     os_window->last_active_tab = os_window->active_tab; os_window->last_num_tabs = os_window->num_tabs; os_window->last_active_window_id = active_window_id;
     os_window->focused_at_last_render = os_window->is_focused;
-    os_window->is_damaged = false;
+    if (os_window->redraw_count) os_window->redraw_count--;
     if (USE_RENDER_FRAMES) request_frame_render(os_window);
 #undef WD
 #undef TD
@@ -821,23 +840,23 @@ no_render_frame_received_recently(OSWindow *w, monotonic_t now, monotonic_t max_
     bool ans = now - w->last_render_frame_received_at > max_wait;
     if (ans && global_state.debug_rendering) {
         if (global_state.is_wayland) {
-            fprintf(stderr, "[%f] No render frame received in %.2f seconds", monotonic_t_to_s_double(now), monotonic_t_to_s_double(max_wait));
+            log_error("No render frame received in %.2f seconds", monotonic_t_to_s_double(max_wait));
         } else  {
-            fprintf(stderr, "[%f] No render frame received in %.2f seconds, re-requesting", monotonic_t_to_s_double(now), monotonic_t_to_s_double(max_wait));
+            log_error("No render frame received in %.2f seconds, re-requesting", monotonic_t_to_s_double(max_wait));
         }
     }
     return ans;
 }
 
 bool
-render_os_window(OSWindow *w, monotonic_t now, bool ignore_render_frames, bool scan_for_animated_images) {
+render_os_window(OSWindow *w, monotonic_t now, bool scan_for_animated_images) {
     if (!w->num_tabs) return false;
     if (!should_os_window_be_rendered(w)) {
         update_os_window_title(w);
         if (w->is_focused) change_menubar_title(w->window_title);
         return false;
     }
-    if (!ignore_render_frames && USE_RENDER_FRAMES && w->render_state != RENDER_FRAME_READY) {
+    if (!w->keep_rendering_till_swap && USE_RENDER_FRAMES && w->render_state != RENDER_FRAME_READY) {
         if (w->render_state == RENDER_FRAME_NOT_REQUESTED || no_render_frame_received_recently(w, now, ms_to_monotonic_t(250ll))) request_frame_render(w);
         // dont respect render frames soon after a resize on Wayland as they cause flicker because
         // we want to fill the newly resized buffer ASAP, not at compositors convenience
@@ -848,7 +867,7 @@ render_os_window(OSWindow *w, monotonic_t now, bool ignore_render_frames, bool s
     w->render_calls++;
     make_os_window_context_current(w);
     if (w->live_resize.in_progress) blank_os_window(w);
-    bool needs_render = w->is_damaged || w->live_resize.in_progress;
+    bool needs_render = w->redraw_count > 0 || w->live_resize.in_progress;
     if (w->viewport_size_dirty) {
         w->clear_count = 0;
         update_surface_size(w->viewport_width, w->viewport_height, 0);
@@ -886,10 +905,17 @@ render(monotonic_t now, bool input_read) {
         // rendering is done in cocoa_os_window_resized()
         if (w->live_resize.in_progress) continue;
 #endif
-        if (!render_os_window(w, now, false, scan_for_animated_images)) {
+        if (!render_os_window(w, now, scan_for_animated_images)) {
             // since we didn't scan the window for animations, force a rescan on next wakeup/render frame
             if (scan_for_animated_images) global_state.check_for_active_animated_images = true;
         }
+        if (w->keep_rendering_till_swap) {
+            debug_rendering("Re-rendering window %llu on the %u attempt since swap did not happen\n", w->id, w->keep_rendering_till_swap);
+            set_maximum_wait(OPT(repaint_delay));
+            w->needs_render = true;
+            w->keep_rendering_till_swap--;
+        }
+
     }
     last_render_at = now;
 #undef TD
@@ -948,14 +974,17 @@ monitor_pid(PyObject *self UNUSED, PyObject *args) {
 
 static void
 report_reaped_pids(void) {
+    static ReapedPID pids[64];
+    size_t i = 0;
     children_mutex(lock);
     if (reaped_pids_count) {
-        for (size_t i = 0; i < reaped_pids_count; i++) {
-            call_boss(on_monitored_pid_death, "ii", (int)reaped_pids[i].pid, reaped_pids[i].status);
+        for (; i < reaped_pids_count && i < arraysz(pids); i++) {
+            pids[i] = reaped_pids[i];
         }
         reaped_pids_count = 0;
     }
     children_mutex(unlock);
+    for (size_t n = 0; n < i; n++) { call_boss(on_monitored_pid_death, "li", (long)pids[n].pid, pids[n].status); }
 }
 
 static void*
@@ -1070,7 +1099,10 @@ process_pending_resizes(monotonic_t now) {
                 update_os_window_viewport(w, true);
                 change_live_resize_state(w, false);
                 zero_at_ptr(&w->live_resize);
-                w->is_damaged = true;  // because the window size should be hidden even if update_os_window_viewport does nothing
+                // because the window size should be hidden even if update_os_window_viewport does nothing
+                // On Wayland some compositors require two redraws after a
+                // resize to actually render correctly (Run kitty -1 --wait-for-os-window-close in sway to reproduce)
+                w->redraw_count = global_state.is_wayland ? 2 : 1;
             }
         }
     }
@@ -1082,8 +1114,11 @@ close_os_window(ChildMonitor *self, OSWindow *os_window) {
     if (os_window->before_fullscreen.is_set && is_os_window_fullscreen(os_window)) {
         w = os_window->before_fullscreen.w; h = os_window->before_fullscreen.h;
     }
+    int x = 0, y = 0;
+    if (os_window->handle && !global_state.is_wayland) glfwGetWindowPos(os_window->handle, &x, &y);
+    bool is_layer_shell = os_window->is_layer_shell;
     destroy_os_window(os_window);
-    call_boss(on_os_window_closed, "Kii", os_window->id, w, h);
+    call_boss(on_os_window_closed, "KiiiiO", os_window->id, x, y, w, h, is_layer_shell ? Py_True : Py_False);
     for (size_t t=0; t < os_window->num_tabs; t++) {
         Tab *tab = os_window->tabs + t;
         for (size_t w = 0; w < tab->num_windows; w++) mark_child_for_close(self, tab->windows[w].id);
@@ -1136,23 +1171,44 @@ process_pending_closes(ChildMonitor *self) {
 // glfw/cocoa. So we use a flag instead.
 static bool cocoa_pending_actions[NUM_COCOA_PENDING_ACTIONS] = {0};
 static bool has_cocoa_pending_actions = false;
+typedef struct cocoa_list { char **items; size_t count, capacity; } cocoa_list;
 typedef struct {
     char* wd;
-    char **open_urls;
-    size_t open_urls_count;
-    size_t open_urls_capacity;
+    cocoa_list open_urls, untracked_notifications;
 } CocoaPendingActionsData;
 static CocoaPendingActionsData cocoa_pending_actions_data = {0};
 
+static void
+cocoa_append_to_pending_list(cocoa_list *array, const char* item) {
+    ensure_space_for(array, items, char*, array->count + 1, capacity, 8, false);
+    array->items[array->count++] = strdup(item);
+}
+
+static void
+cocoa_free_pending_list(cocoa_list *array) {
+    for (size_t i = 0; i < array->count; i++) free(array->items[i]);
+    free(array->items); zero_at_ptr(array);
+}
+
+static void
+cocoa_free_actions_data(void) {
+    if (cocoa_pending_actions_data.wd) { free(cocoa_pending_actions_data.wd); cocoa_pending_actions_data.wd = NULL; }
+    cocoa_free_pending_list(&cocoa_pending_actions_data.open_urls);
+    cocoa_free_pending_list(&cocoa_pending_actions_data.untracked_notifications);
+}
+
 void
 set_cocoa_pending_action(CocoaPendingAction action, const char *data) {
     if (data) {
-        if (action == LAUNCH_URLS) {
-            ensure_space_for(&cocoa_pending_actions_data, open_urls, char*, cocoa_pending_actions_data.open_urls_count + 8, open_urls_capacity, 8, true);
-            cocoa_pending_actions_data.open_urls[cocoa_pending_actions_data.open_urls_count++] = strdup(data);
-        } else {
-            if (cocoa_pending_actions_data.wd) free(cocoa_pending_actions_data.wd);
-            cocoa_pending_actions_data.wd = strdup(data);
+        switch(action) {
+            case LAUNCH_URLS:
+                cocoa_append_to_pending_list(&cocoa_pending_actions_data.open_urls, data); break;
+            case COCOA_NOTIFICATION_UNTRACKED:
+                cocoa_append_to_pending_list(&cocoa_pending_actions_data.untracked_notifications, data); break;
+            default:
+                if (cocoa_pending_actions_data.wd) free(cocoa_pending_actions_data.wd);
+                cocoa_pending_actions_data.wd = strdup(data);
+                break;
         }
     }
     cocoa_pending_actions[action] = true;
@@ -1176,6 +1232,8 @@ process_cocoa_pending_actions(void) {
     if (cocoa_pending_actions[CLOSE_WINDOW]) { call_boss(close_window, NULL); }
     if (cocoa_pending_actions[RESET_TERMINAL]) { call_boss(clear_terminal, "sO", "reset", Py_True ); }
     if (cocoa_pending_actions[CLEAR_TERMINAL_AND_SCROLLBACK]) { call_boss(clear_terminal, "sO", "to_cursor", Py_True ); }
+    if (cocoa_pending_actions[CLEAR_SCROLLBACK]) { call_boss(clear_terminal, "sO", "scrollback", Py_True ); }
+    if (cocoa_pending_actions[CLEAR_SCREEN]) { call_boss(clear_terminal, "sO", "to_cursor_scroll", Py_True ); }
     if (cocoa_pending_actions[RELOAD_CONFIG]) { call_boss(load_config_file, NULL); }
     if (cocoa_pending_actions[TOGGLE_MACOS_SECURE_KEYBOARD_ENTRY]) { call_boss(toggle_macos_secure_keyboard_entry, NULL); }
     if (cocoa_pending_actions[TOGGLE_FULLSCREEN]) { call_boss(toggle_fullscreen, NULL); }
@@ -1191,16 +1249,25 @@ process_cocoa_pending_actions(void) {
         free(cocoa_pending_actions_data.wd);
         cocoa_pending_actions_data.wd = NULL;
     }
-    if (cocoa_pending_actions_data.open_urls_count) {
-        for (unsigned cpa = 0; cpa < cocoa_pending_actions_data.open_urls_count; cpa++) {
-            if (cocoa_pending_actions_data.open_urls[cpa]) {
-                call_boss(launch_urls, "s", cocoa_pending_actions_data.open_urls[cpa]);
-                free(cocoa_pending_actions_data.open_urls[cpa]);
-                cocoa_pending_actions_data.open_urls[cpa] = NULL;
-            }
+    for (unsigned cpa = 0; cpa < cocoa_pending_actions_data.open_urls.count; cpa++) {
+        if (cocoa_pending_actions_data.open_urls.items[cpa]) {
+            call_boss(launch_urls, "s", cocoa_pending_actions_data.open_urls.items[cpa]);
+            free(cocoa_pending_actions_data.open_urls.items[cpa]);
+            cocoa_pending_actions_data.open_urls.items[cpa] = NULL;
         }
-        cocoa_pending_actions_data.open_urls_count = 0;
     }
+    cocoa_pending_actions_data.open_urls.count = 0;
+
+    for (unsigned cpa = 0; cpa < cocoa_pending_actions_data.untracked_notifications.count; cpa++) {
+        if (cocoa_pending_actions_data.untracked_notifications.items[cpa]) {
+            cocoa_report_live_notifications(cocoa_pending_actions_data.untracked_notifications.items[cpa]);
+            free(cocoa_pending_actions_data.untracked_notifications.items[cpa]);
+            cocoa_pending_actions_data.untracked_notifications.items[cpa] = NULL;
+        }
+    }
+    cocoa_pending_actions_data.untracked_notifications.count = 0;
+
+
     memset(cocoa_pending_actions, 0, sizeof(cocoa_pending_actions));
     has_cocoa_pending_actions = false;
 
@@ -1258,13 +1325,7 @@ main_loop(ChildMonitor *self, PyObject *a UNUSED) {
     state_check_timer = add_main_loop_timer(1000, true, do_state_check, self, NULL);
     run_main_loop(process_global_state, self);
 #ifdef __APPLE__
-    if (cocoa_pending_actions_data.wd) { free(cocoa_pending_actions_data.wd); cocoa_pending_actions_data.wd = NULL; }
-    if (cocoa_pending_actions_data.open_urls) {
-        for (unsigned cpa = 0; cpa < cocoa_pending_actions_data.open_urls_count; cpa++) {
-            if (cocoa_pending_actions_data.open_urls[cpa]) free(cocoa_pending_actions_data.open_urls[cpa]);
-        }
-        free(cocoa_pending_actions_data.open_urls); cocoa_pending_actions_data.open_urls = NULL;
-    }
+    cocoa_free_actions_data();
 #endif
     if (PyErr_Occurred()) return NULL;
     Py_RETURN_NONE;
@@ -1625,6 +1686,20 @@ add_peer(int peer, bool is_remote_control_peer) {
     return ans;
 }
 
+static bool
+getpeerid(int fd, uid_t *euid, gid_t *egid) {
+#ifdef __linux__
+    struct ucred cr;
+    socklen_t sz = sizeof(cr);
+    if (getsockopt(fd, SOL_SOCKET, SO_PEERCRED, &cr, &sz) != 0) return false;
+    *euid = cr.uid; *egid = cr.gid;
+#else
+    if (getpeereid(fd, euid, egid) != 0) return false;
+#endif
+    return true;
+}
+
+
 static bool
 accept_peer(int listen_fd, bool shutting_down, bool is_remote_control_peer) {
     int peer = accept(listen_fd, NULL, NULL);
@@ -1633,6 +1708,21 @@ accept_peer(int listen_fd, bool shutting_down, bool is_remote_control_peer) {
         if (!shutting_down) perror("accept() on talk socket failed!");
         return false;
     }
+    if (verify_peer_uid) {
+        uid_t peer_uid; gid_t peer_gid;
+        if (!getpeerid(peer, &peer_uid, &peer_gid)) {
+            log_error("Denying access to peer because failed to get uid and gid for peer: %d with error: %s", peer, strerror(errno));
+            shutdown(peer, SHUT_RDWR);
+            safe_close(peer, __FILE__, __LINE__);
+            return true;
+        }
+        if (peer_uid != geteuid()) {
+            log_error("Denying access to peer because its uid (%d) does not match our uid (%d)", peer_uid, geteuid());
+            shutdown(peer, SHUT_RDWR);
+            safe_close(peer, __FILE__, __LINE__);
+            return true;
+        }
+    }
     add_peer(peer, is_remote_control_peer);
     return true;
 }
@@ -1964,28 +2054,8 @@ send_data_to_peer(PyObject *self UNUSED, PyObject *args) {
     Py_RETURN_NONE;
 }
 
-static PyObject *
-random_unix_socket(PyObject *self UNUSED, PyObject *args UNUSED) {
-#ifndef SO_PASSCRED
-    errno = ENOTSUP;
-    return PyErr_SetFromErrno(PyExc_OSError);
-#else
-    int fd, optval = 1;
-    struct sockaddr_un bind_addr = {.sun_family=AF_UNIX};
-    fd = socket(AF_UNIX, SOCK_STREAM, 0);
-    if (fd < 0) return PyErr_SetFromErrno(PyExc_OSError);
-    if (setsockopt(fd, SOL_SOCKET, SO_PASSCRED, &optval, sizeof optval) != 0) goto fail;
-    if (bind(fd, (struct sockaddr *)&bind_addr, sizeof(sa_family_t)) != 0) goto fail;
-    return PyLong_FromLong((long)fd);
-fail:
-    safe_close(fd, __FILE__, __LINE__);
-    return PyErr_SetFromErrno(PyExc_OSError);
-#endif
-}
-
 static PyMethodDef module_methods[] = {
     METHODB(safe_pipe, METH_VARARGS),
-    METHODB(random_unix_socket, METH_NOARGS),
     {"add_timer", (PyCFunction)add_python_timer, METH_VARARGS, ""},
     {"remove_timer", (PyCFunction)remove_python_timer, METH_VARARGS, ""},
     METHODB(monitor_pid, METH_VARARGS),
