@@ -7,7 +7,7 @@ from collections import defaultdict
 from collections.abc import Generator, Sequence
 from contextlib import contextmanager, suppress
 from itertools import count
-from typing import TYPE_CHECKING, DefaultDict, Iterable, Optional, TypedDict
+from typing import TYPE_CHECKING, DefaultDict, Iterable, Mapping, Optional, TypedDict
 
 import kitty.fast_data_types as fast_data_types
 
@@ -145,8 +145,8 @@ def environ_of_process(pid: int) -> dict[str, str]:
     return parse_environ_block(_environ_of_process(pid))
 
 
-def process_env() -> dict[str, str]:
-    ans = dict(os.environ)
+def process_env(env: Mapping[str, str] | None = None) -> dict[str, str]:
+    ans = dict(os.environ if env is None else env)
     ssl_env_var = getattr(sys, 'kitty_ssl_env_var', None)
     if ssl_env_var is not None:
         ans.pop(ssl_env_var, None)
@@ -423,13 +423,10 @@ class Child:
             sid = session_id(gmap.get(foreground_process_group_id, ()))
             if sid < 0:
                 return []
-            ans = []
+            ans: list[ProcessDesc] = []
             for grp_id, pids in gmap.items():
-                if grp_id == foreground_process_group_id:
-                    continue
-                if session_id(pids) == sid:
-                    for pid in pids:
-                        ans.append(self.process_desc(pid))
+                if grp_id != foreground_process_group_id and session_id(pids) == sid:
+                    ans.extend(map(self.process_desc, pids))
             return ans
         except Exception:
             return []
