@@ -700,7 +700,12 @@ enter_event(int modifiers) {
     // On cocoa there is no way to configure the window manager to
     // focus windows on mouse enter, so we do it ourselves
     if (OPT(focus_follows_mouse) && !global_state.callback_os_window->is_focused) {
+        id_type wid = global_state.callback_os_window->id;
         focus_os_window(global_state.callback_os_window, false, NULL);
+        if (!global_state.callback_os_window) {
+            global_state.callback_os_window = os_window_for_id(wid);
+            if (!global_state.callback_os_window) return;
+        }
     }
 #endif
     // If the mouse is grabbed send a move event to update the cursor position
@@ -739,6 +744,7 @@ typedef enum MouseSelectionType {
     MOUSE_SELECTION_LINE_FROM_POINT,
     MOUSE_SELECTION_WORD_AND_LINE_FROM_POINT,
     MOUSE_SELECTION_MOVE_END,
+    MOUSE_SELECTION_UPTO_SURROUNDING_WHITESPACE,
 } MouseSelectionType;
 
 
@@ -778,6 +784,9 @@ mouse_selection(Window *w, int code, int button) {
         case MOUSE_SELECTION_MOVE_END:
             extend_selection(w, false, false);
             break;
+        case MOUSE_SELECTION_UPTO_SURROUNDING_WHITESPACE:
+            // TODO: Implement me for people migrating from urxvt
+            break;
     }
     set_mouse_cursor_when_dragging(screen);
 #undef S
@@ -1110,6 +1119,7 @@ init_mouse(PyObject *module) {
     PyModule_AddIntMacro(module, MOUSE_SELECTION_LINE_FROM_POINT);
     PyModule_AddIntMacro(module, MOUSE_SELECTION_WORD_AND_LINE_FROM_POINT);
     PyModule_AddIntMacro(module, MOUSE_SELECTION_MOVE_END);
+    PyModule_AddIntMacro(module, MOUSE_SELECTION_UPTO_SURROUNDING_WHITESPACE);
     if (PyModule_AddFunctions(module, module_methods) != 0) return false;
     return true;
 }
