@@ -6,18 +6,23 @@
 
 #pragma once
 #include "data-types.h"
+#include "animation.h"
 #include "screen.h"
 #include "monotonic.h"
 #include "window_logo.h"
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wpedantic"
+#include <hb.h>
+#pragma GCC diagnostic pop
 
 #define OPT(name) global_state.opts.name
 #define debug_rendering(...) if (global_state.debug_rendering) { timed_debug_print(__VA_ARGS__); }
 #define debug_input(...) if (OPT(debug_keyboard)) { timed_debug_print(__VA_ARGS__); }
 #define debug_fonts(...) if (global_state.debug_font_fallback) { timed_debug_print(__VA_ARGS__); }
 
-typedef enum { LEFT_EDGE, TOP_EDGE, RIGHT_EDGE, BOTTOM_EDGE } Edge;
+typedef enum { LEFT_EDGE = 1, TOP_EDGE = 2, RIGHT_EDGE = 4, BOTTOM_EDGE = 8 } Edge;
 typedef enum { REPEAT_MIRROR, REPEAT_CLAMP, REPEAT_DEFAULT } RepeatStrategy;
-typedef enum { WINDOW_NORMAL, WINDOW_FULLSCREEN, WINDOW_MAXIMIZED, WINDOW_MINIMIZED } WindowState;
+typedef enum { WINDOW_NORMAL, WINDOW_FULLSCREEN, WINDOW_MAXIMIZED, WINDOW_MINIMIZED, WINDOW_HIDDEN } WindowState;
 
 typedef struct {
     char_type string[16];
@@ -34,20 +39,28 @@ struct MenuItem {
 };
 
 typedef struct {
-    monotonic_t visual_bell_duration, cursor_blink_interval, cursor_stop_blinking_after, mouse_hide_wait, click_interval;
+    monotonic_t visual_bell_duration, cursor_blink_interval, cursor_stop_blinking_after, click_interval;
+    struct {
+        monotonic_t hide_wait, unhide_wait;
+        int unhide_threshold;
+        bool scroll_unhide;
+    } mouse_hide;
     double wheel_scroll_multiplier, touch_scroll_multiplier;
     int wheel_scroll_min_lines;
     bool enable_audio_bell;
-    CursorShape cursor_shape;
+    CursorShape cursor_shape, cursor_shape_unfocused;
     float cursor_beam_thickness;
     float cursor_underline_thickness;
+    monotonic_t cursor_trail;
+    float cursor_trail_decay_fast;
+    float cursor_trail_decay_slow;
+    float cursor_trail_start_threshold;
     unsigned int url_style;
     unsigned int scrollback_pager_history_size;
     bool scrollback_fill_enlarged_window;
     char_type *select_by_word_characters;
     char_type *select_by_word_characters_forward;
     color_type url_color, background, foreground, active_border_color, inactive_border_color, bell_border_color, tab_bar_background, tab_bar_margin_color;
-    color_type mark1_foreground, mark1_background, mark2_foreground, mark2_background, mark3_foreground, mark3_background;
     monotonic_t repaint_delay, input_delay;
     bool focus_follows_mouse;
     unsigned int hide_window_decorations;
@@ -65,6 +78,7 @@ typedef struct {
     ImageAnchorPosition window_logo_position;
     bool background_image_linear;
     float background_tint, background_tint_gaps, window_logo_alpha;
+    struct { float width, height; } window_logo_scale;
 
     bool dynamic_background_opacity;
     float inactive_text_alpha;
@@ -81,7 +95,7 @@ typedef struct {
     struct { monotonic_t on_end, on_pause; } resize_debounce_time;
     MouseShape pointer_shape_when_grabbed;
     MouseShape default_pointer_shape;
-    MouseShape pointer_shape_when_dragging;
+    MouseShape pointer_shape_when_dragging, pointer_shape_when_dragging_rectangle;
     struct {
         UrlPrefix *values;
         size_t num, max_prefix_len;
@@ -105,6 +119,18 @@ typedef struct {
     unsigned long wayland_titlebar_color;
     struct { struct MenuItem *entries; size_t count; } global_menu;
     bool wayland_enable_ime;
+    struct {
+        size_t num;
+        struct {
+            const char *psname;
+            size_t num;
+            hb_feature_t *features;
+        } *entries;
+    } font_features;
+    struct { Animation *cursor, *visual_bell; } animation;
+    unsigned undercurl_style;
+    struct { float thickness; int unit; } underline_exclusion;
+    float box_drawing_scale[4];
 } Options;
 
 typedef struct WindowLogoRenderData {
@@ -144,6 +170,17 @@ typedef struct MousePosition {
     bool in_left_half_of_cell;
 } MousePosition;
 
+typedef struct PendingClick {
+    id_type window_id;
+    int button, count, modifiers;
+    bool grabbed;
+    monotonic_t at;
+    MousePosition mouse_pos;
+    unsigned long press_num;
+    double radius_for_multiclick;
+} PendingClick;
+
+
 typedef struct WindowBarData {
     unsigned width, height;
     uint8_t *buf;
@@ -154,7 +191,8 @@ typedef struct WindowBarData {
 
 typedef struct {
     id_type id;
-    bool visible, cursor_visible_at_last_render;
+    bool visible;
+    float cursor_opacity_at_last_render;
     CursorShape last_cursor_shape;
     PyObject *title;
     WindowRenderData render_data;
@@ -168,6 +206,16 @@ typedef struct {
     monotonic_t last_drag_scroll_at;
     uint32_t last_special_key_pressed;
     WindowBarData title_bar_data, url_target_bar_data;
+    id_type redirect_keys_to_overlay;
+    struct {
+        bool enabled;
+        void *key_data;
+        size_t count, capacity;
+    } buffered_keys;
+    struct {
+        PendingClick *clicks;
+        size_t num, capacity;
+    } pending_clicks;
 } Window;
 
 typedef struct {
@@ -182,11 +230,22 @@ typedef struct {
     ssize_t vao_idx;
 } BorderRects;
 
+typedef struct {
+    bool needs_render;
+    monotonic_t updated_at;
+    float opacity;
+    float corner_x[4];
+    float corner_y[4];
+    float cursor_edge_x[2];
+    float cursor_edge_y[2];
+} CursorTrail;
+
 typedef struct {
     id_type id;
     unsigned int active_window, num_windows, capacity;
     Window *windows;
     BorderRects border_rects;
+    CursorTrail cursor_trail;
 } Tab;
 
 enum RENDER_STATE { RENDER_FRAME_NOT_REQUESTED, RENDER_FRAME_REQUESTED, RENDER_FRAME_READY };
@@ -226,13 +285,15 @@ typedef struct {
     BackgroundImage *bgimage;
     unsigned int active_tab, num_tabs, capacity, last_active_tab, last_num_tabs, last_active_window_id;
     bool focused_at_last_render, needs_render;
+    unsigned keep_rendering_till_swap;
     WindowRenderData tab_bar_render_data;
     struct {
         color_type left, right;
     } tab_bar_edge_color;
     bool tab_bar_data_updated;
     bool is_focused;
-    monotonic_t cursor_blink_zero_time, last_mouse_activity_at;
+    monotonic_t cursor_blink_zero_time, last_mouse_activity_at, mouse_activate_deadline;
+    int mouse_show_threshold;
     bool has_received_cursor_pos_event;
     double mouse_x, mouse_y;
     bool mouse_button_pressed[32];
@@ -241,8 +302,8 @@ typedef struct {
     bool viewport_size_dirty, viewport_updated_at_least_once;
     monotonic_t viewport_resized_at;
     LiveResizeInfo live_resize;
-    bool has_pending_resizes, is_semi_transparent, shown_once, is_damaged, ignore_resize_events;
-    unsigned int clear_count;
+    bool has_pending_resizes, is_semi_transparent, shown_once, ignore_resize_events;
+    unsigned int clear_count, redraw_count;
     WindowChromeState last_window_chrome;
     float background_opacity;
     FONTS_DATA_HANDLE fonts_data;
@@ -252,6 +313,7 @@ typedef struct {
     uint64_t render_calls;
     id_type last_focused_counter;
     CloseRequest close_request;
+    bool is_layer_shell, hide_on_focus_loss;
 } OSWindow;
 
 
@@ -264,7 +326,7 @@ typedef struct {
     OSWindow *os_windows;
     size_t num_os_windows, capacity;
     OSWindow *callback_os_window;
-    bool is_wayland;
+    bool is_wayland, is_apple;
     bool has_render_frames;
     bool debug_rendering, debug_font_fallback;
     bool has_pending_resizes, has_pending_closes;
@@ -276,6 +338,7 @@ typedef struct {
     bool redirect_mouse_handling;
     WindowLogoTable *all_window_logos;
     int gl_version;
+    PyObject *options_object;
 } GlobalState;
 
 extern GlobalState global_state;
@@ -286,6 +349,13 @@ extern GlobalState global_state;
     else Py_DECREF(cret_); \
 }
 
+static inline void
+sprite_index_to_pos(unsigned idx, unsigned xnum, unsigned ynum, unsigned *x, unsigned *y, unsigned *z) {
+    div_t r = div(idx & 0x7fffffff, ynum * xnum), r2 = div(r.rem, xnum);
+    *z = r.quot; *y = r2.quot; *x = r2.rem;
+}
+
+
 void gl_init(void);
 void remove_vao(ssize_t vao_idx);
 bool remove_os_window(id_type os_window_id);
@@ -321,47 +391,20 @@ ssize_t create_border_vao(void);
 bool send_cell_data_to_gpu(ssize_t, float, float, float, float, Screen *, OSWindow *);
 void draw_cells(ssize_t, const WindowRenderData*, OSWindow *, bool, bool, bool, Window*);
 void draw_centered_alpha_mask(OSWindow *w, size_t screen_width, size_t screen_height, size_t width, size_t height, uint8_t *canvas, float);
+void draw_cursor_trail(CursorTrail *trail, Window *active_window);
+bool update_cursor_trail(CursorTrail *ct, Window *w, monotonic_t now, OSWindow *os_window);
 void update_surface_size(int, int, uint32_t);
 void free_texture(uint32_t*);
 void free_framebuffer(uint32_t*);
 void send_image_to_gpu(uint32_t*, const void*, int32_t, int32_t, bool, bool, bool, RepeatStrategy);
-void send_sprite_to_gpu(FONTS_DATA_HANDLE fg, unsigned int, unsigned int, unsigned int, pixel*);
+void send_sprite_to_gpu(FONTS_DATA_HANDLE fg, sprite_index, pixel*, sprite_index);
 void blank_canvas(float, color_type);
 void blank_os_window(OSWindow *);
 void set_os_window_chrome(OSWindow *w);
 FONTS_DATA_HANDLE load_fonts_data(double, double, double);
 void send_prerendered_sprites_for_window(OSWindow *w);
 #ifdef __APPLE__
-void get_cocoa_key_equivalent(uint32_t, int, char *key, size_t key_sz, int*);
-typedef enum {
-    PREFERENCES_WINDOW,
-    NEW_OS_WINDOW,
-    NEW_OS_WINDOW_WITH_WD,
-    NEW_TAB_WITH_WD,
-    CLOSE_OS_WINDOW,
-    CLOSE_TAB,
-    NEW_TAB,
-    NEXT_TAB,
-    PREVIOUS_TAB,
-    DETACH_TAB,
-    LAUNCH_URLS,
-    NEW_WINDOW,
-    CLOSE_WINDOW,
-    RESET_TERMINAL,
-    CLEAR_TERMINAL_AND_SCROLLBACK,
-    RELOAD_CONFIG,
-    TOGGLE_MACOS_SECURE_KEYBOARD_ENTRY,
-    TOGGLE_FULLSCREEN,
-    OPEN_KITTY_WEBSITE,
-    HIDE,
-    HIDE_OTHERS,
-    MINIMIZE,
-    QUIT,
-    USER_MENU_ACTION,
-
-    NUM_COCOA_PENDING_ACTIONS
-} CocoaPendingAction;
-void set_cocoa_pending_action(CocoaPendingAction action, const char*);
+#include "cocoa_window.h"
 #endif
 void request_frame_render(OSWindow *w);
 void request_tick_callback(void);
@@ -372,7 +415,7 @@ void remove_main_loop_timer(id_type timer_id);
 void update_main_loop_timer(id_type timer_id, monotonic_t interval, bool enabled);
 void run_main_loop(tick_callback_fun, void*);
 void stop_main_loop(void);
-void os_window_update_size_increments(OSWindow *window);
+void on_os_window_font_size_change(OSWindow *window, double new_sz);
 void set_os_window_title_from_window(Window *w, OSWindow *os_window);
 void update_os_window_title(OSWindow *os_window);
 void fake_scroll(Window *w, int amount, bool upwards);
@@ -383,8 +426,8 @@ bool mouse_select_cmd_output(Window *w);
 bool move_cursor_to_mouse_if_at_shell_prompt(Window *w);
 void mouse_selection(Window *w, int code, int button);
 const char* format_mods(unsigned mods);
-void send_pending_click_to_window_id(id_type, void*);
-void send_pending_click_to_window(Window*, void*);
+void dispatch_pending_clicks(id_type, void*);
+void send_pending_click_to_window(Window*, int);
 void get_platform_dependent_config_values(void *glfw_window);
 bool draw_window_title(OSWindow *window, const char *text, color_type fg, color_type bg, uint8_t *output_buf, size_t width, size_t height);
 uint8_t* draw_single_ascii_char(const char ch, size_t *result_width, size_t *result_height);
@@ -395,6 +438,7 @@ bool update_ime_position_for_window(id_type window_id, bool force, int update_fo
 void set_ignore_os_keyboard_processing(bool enabled);
 void update_menu_bar_title(PyObject *title UNUSED);
 void change_live_resize_state(OSWindow*, bool);
-bool render_os_window(OSWindow *w, monotonic_t now, bool ignore_render_frames, bool scan_for_animated_images);
+bool render_os_window(OSWindow *w, monotonic_t now, bool scan_for_animated_images);
 void update_mouse_pointer_shape(void);
 void adjust_window_size_for_csd(OSWindow *w, int width, int height, int *adjusted_width, int *adjusted_height);
+void dispatch_buffered_keys(Window *w);
