@@ -53,7 +53,7 @@ from .layout.interface import create_layout_object_for, evict_cached_layouts
 from .progress import ProgressState
 from .tab_bar import TabBar, TabBarData
 from .types import ac
-from .typing import EdgeLiteral, SessionTab, SessionType, TypedDict
+from .typing_compat import EdgeLiteral, SessionTab, SessionType, TypedDict
 from .utils import cmdline_for_hold, log_error, platform_window_id, resolved_shell, shlex_split, which
 from .window import CwdRequest, Watchers, Window, WindowDict
 from .window_list import WindowList
@@ -126,6 +126,7 @@ class Tab:  # {{{
     confirm_close_window_id: int = 0
     num_of_windows_with_progress: int = 0
     total_progress: int = 0
+    has_indeterminate_progress: bool = False
     last_focused_window_with_progress_id: int = 0
 
     def __init__(
@@ -169,6 +170,7 @@ class Tab:  # {{{
         self.num_of_windows_with_progress = 0
         self.total_progress = 0
         self.last_focused_window_with_progress_id = 0
+        self.has_indeterminate_progress = False
         focused_at = 0.
         for window in self:
             p = window.progress
@@ -177,10 +179,15 @@ class Tab:  # {{{
             if p.state in (ProgressState.set, ProgressState.paused):
                 self.total_progress += p.percent
                 self.num_of_windows_with_progress += 1
+            elif p.state is ProgressState.indeterminate:
+                self.has_indeterminate_progress = True
             if window.last_focused_at > focused_at or (not window.last_focused_at and window.id > self.last_focused_window_with_progress_id):
                 focused_at = window.last_focused_at
                 self.last_focused_window_with_progress_id = window.id
         self.mark_tab_bar_dirty()
+        tm = self.tab_manager_ref()
+        if tm is not None:
+            tm.update_progress()
 
     def has_single_window_visible(self) -> bool:
         if self.current_layout.only_active_window_visible:
@@ -444,7 +451,7 @@ class Tab:  # {{{
         if w is not None and self.resize_window_by(
                 w.id, increment, is_horizontal) is not None:
             if get_options().enable_audio_bell:
-                ring_bell()
+                ring_bell(self.os_window_id)
 
     @ac('win', 'Reset window sizes undoing any dynamic resizing of windows')
     def reset_window_sizes(self) -> None:
@@ -456,7 +463,7 @@ class Tab:  # {{{
         ret = self.current_layout.layout_action(action_name, args, self.windows)
         if ret is None:
             if get_options().enable_audio_bell:
-                ring_bell()
+                ring_bell(self.os_window_id)
             return
         self.relayout()
 
@@ -910,6 +917,9 @@ class Tab:  # {{{
 class TabManager:  # {{{
 
     confirm_close_window_id: int = 0
+    num_of_windows_with_progress: int = 0
+    total_progress: int = 0
+    has_indeterminate_progress: bool = False
 
     def __init__(self, os_window_id: int, args: CLIOptions, wm_class: str, wm_name: str, startup_session: SessionType | None = None):
         self.os_window_id = os_window_id
@@ -1205,6 +1215,10 @@ class TabManager:  # {{{
                     return idx
             return -1
 
+        def remove_from_end_of_active_history(idx: int) -> None:
+            while self.active_tab_history and idx_for_id(self.active_tab_history[-1]) == idx:
+                self.active_tab_history.pop()
+
         if active_tab_needs_to_change:
             next_active_tab = -1
             tss = get_options().tab_switch_strategy
@@ -1214,10 +1228,13 @@ class TabManager:  # {{{
                     next_active_tab = idx_for_id(tab_id)
             elif tss == 'left':
                 next_active_tab = max(0, self.active_tab_idx - 1)
+                remove_from_end_of_active_history(next_active_tab)
             elif tss == 'right':
                 next_active_tab = min(self.active_tab_idx, len(self.tabs) - 1)
+                remove_from_end_of_active_history(next_active_tab)
             elif tss == 'last':
                 next_active_tab = len(self.tabs) - 1
+                remove_from_end_of_active_history(next_active_tab)
 
             if next_active_tab < 0:
                 next_active_tab = max(0, min(self.active_tab_idx, len(self.tabs) - 1))
@@ -1283,6 +1300,18 @@ class TabManager:  # {{{
         if len(self.recent_mouse_events) > 5:
             self.recent_mouse_events.popleft()
 
+    def update_progress(self) -> None:
+        self.num_of_windows_with_progress = 0
+        self.total_progress = 0
+        self.has_indeterminate_progress = False
+        for tab in self:
+            if tab.num_of_windows_with_progress:
+                self.total_progress += tab.total_progress
+                self.num_of_windows_with_progress += tab.num_of_windows_with_progress
+            if tab.has_indeterminate_progress:
+                self.has_indeterminate_progress = True
+        get_boss().update_progress_in_dock()
+
     @property
     def tab_bar_rects(self) -> tuple[Border, ...]:
         return self.tab_bar.blank_rects if self.tab_bar_should_be_visible else ()
