@@ -92,7 +92,7 @@ from .progress import Progress
 from .rgb import to_color
 from .terminfo import get_capabilities
 from .types import MouseEvent, OverlayType, WindowGeometry, ac, run_once
-from .typing import BossType, ChildType, EdgeLiteral, TabType, TypedDict
+from .typing_compat import BossType, ChildType, EdgeLiteral, TabType, TypedDict
 from .utils import (
     color_as_int,
     docs_url,
@@ -242,6 +242,7 @@ class WindowDict(TypedDict):
     user_vars: dict[str, str]
     at_prompt: bool
     created_at: int
+    in_alternate_screen: bool
 
 
 class PipeData(TypedDict):
@@ -635,6 +636,7 @@ class Window:
         self.last_resized_at = 0.
         self.started_at = monotonic()
         self.created_at = time_ns()
+        self.clear_progress_timer: int = 0
         self.current_remote_data: list[str] = []
         self.current_mouse_event_button = 0
         self.current_clipboard_read_ask: bool | None = None
@@ -780,6 +782,7 @@ class Window:
             'columns': self.screen.columns,
             'user_vars': self.user_vars,
             'created_at': self.created_at,
+            'in_alternate_screen': self.screen.is_using_alternate_linebuf(),
         }
 
     def serialize_state(self) -> dict[str, Any]:
@@ -1130,11 +1133,14 @@ class Window:
 
     def clear_progress_if_needed(self, timer_id: int | None = None) -> None:
         # Clear stuck or completed progress
+        if timer_id is not None:  # this is a timer callback
+            self.clear_progress_timer = 0
         if self.progress.clear_progress():
             if (tab := self.tabref()) is not None:
                 tab.update_progress()
         else:
-            add_timer(self.clear_progress_if_needed, 1.0, False)
+            if not self.clear_progress_timer:
+                self.clear_progress_timer = add_timer(self.clear_progress_if_needed, 1.0, False)
 
     def on_mouse_event(self, event: dict[str, Any]) -> bool:
         event['mods'] = event.get('mods', 0) & mod_mask
