@@ -31,9 +31,9 @@ class TestBuild(BaseTest):
         del fdt, rsync
 
     def test_loading_shaders(self) -> None:
-        from kitty.utils import load_shaders
-        for name in 'cell border bgimage tint blit graphics'.split():
-            load_shaders(name)
+        from kitty.shaders import Program
+        for name in 'cell border bgimage tint graphics'.split():
+            Program(name)
 
     def test_glfw_modules(self) -> None:
         from kitty.constants import glfw_path, is_macos
@@ -54,11 +54,13 @@ class TestBuild(BaseTest):
         self.assertGreater(len(names), 8)
 
     def test_filesystem_locations(self) -> None:
-        from kitty.constants import local_docs, logo_png_file, shell_integration_dir, terminfo_dir
+        from kitty.constants import fonts_dir, local_docs, logo_png_file, shell_integration_dir, terminfo_dir
         zsh = os.path.join(shell_integration_dir, 'zsh')
         self.assertTrue(os.path.isdir(terminfo_dir), f'Terminfo dir: {terminfo_dir}')
         self.assertTrue(os.path.exists(logo_png_file), f'Logo file: {logo_png_file}')
         self.assertTrue(os.path.exists(zsh), f'Shell integration: {zsh}')
+        nsfm = os.path.join(fonts_dir, 'SymbolsNerdFontMono-Regular.ttf')
+        self.assertTrue(os.path.exists(nsfm), f'Logo file: {nsfm}')
 
         def is_executable(x):
             mode = os.stat(x).st_mode
