@@ -98,6 +98,11 @@ def test_multicell(self: TestMulticell) -> None:
     def assert_cursor_at(x, y):
         self.ae((s.cursor.x, s.cursor.y), (x, y))
 
+    s = self.create_screen(cols=8, lines=4)
+    s.draw('飛青進服三上')
+    s.resize(s.lines, 5)
+    self.ae('飛青', str(s.line(0)))
+
     s = self.create_screen(cols=6, lines=6)
 
     # Test basic multicell drawing
@@ -130,12 +135,22 @@ def test_multicell(self: TestMulticell) -> None:
         for x in range(1, 3):
             comb(x, y)
     comb(0, 1)
+    s.reset()
+    multicell(s, 'aüa', scale=2)
+    self.ae(s.cursor.x, 6)
     s = self.create_screen(cols=7 * 7, lines=7)
     multicell(s, 'a', scale=7, width=7)
     for y in range(s.lines):
         for x in range(s.columns):
             ac(x, y, is_multicell=True, x=x, y=y)
 
+    # Test zero width roundtripping
+    for preserved in '\xad\u200b\u2060':
+        s.reset()
+        multicell(s, f'|{preserved}|', scale=2)
+        assert_cursor_at(4, 0)
+        ac(0, 0, text=f'|{preserved}')
+
     # Test wrapping
     s = self.create_screen(cols=6, lines=6)
     s.draw('x' * (s.columns - 1))
