@@ -7,7 +7,8 @@ import (
 	"os"
 	"time"
 
-	"kitty/tools/tui/loop"
+	"github.com/kovidgoyal/kitty/tools/tui/loop"
+	"github.com/kovidgoyal/kitty/tools/utils"
 )
 
 type stream_response struct {
@@ -26,9 +27,15 @@ func is_stream_response(serialized_response []byte) bool {
 
 func do_chunked_io(io_data *rc_io_data) (serialized_response []byte, err error) {
 	serialized_response = make([]byte, 0)
-	lp, err := loop.New(loop.NoAlternateScreen, loop.NoRestoreColors, loop.OnlyDisambiguateKeys)
+	// we cant do inbandresize notification as in the --no-response case the
+	// command can cause a resize and the loop can quit before the notification
+	// arrives, leading to the notification being sent to whatever is executed
+	// after us.
+	lp, err := loop.New(loop.NoAlternateScreen, loop.NoRestoreColors, loop.NoInBandResizeNotifications)
 	if io_data.on_key_event != nil {
 		lp.FullKeyboardProtocol()
+	} else {
+		lp.NoKeyboardStateChange()
 	}
 	if err != nil {
 		return
@@ -109,7 +116,7 @@ func do_chunked_io(io_data *rc_io_data) (serialized_response []byte, err error)
 			return err
 		}
 		if len(chunk) == 0 {
-			state = WAITING_FOR_RESPONSE
+			state = utils.IfElse(state == BEFORE_FIRST_ESCAPE_CODE_SENT && wants_streaming, WAITING_FOR_STREAMING_RESPONSE, WAITING_FOR_RESPONSE)
 			transition_to_read()
 		} else {
 			queue_escape_code(chunk)
