@@ -3,6 +3,7 @@
 package edit_in_kitty
 
 import (
+	"bytes"
 	"encoding/base64"
 	"fmt"
 	"io"
@@ -13,11 +14,11 @@ import (
 
 	"golang.org/x/sys/unix"
 
-	"kitty/tools/cli"
-	"kitty/tools/tui"
-	"kitty/tools/tui/loop"
-	"kitty/tools/utils"
-	"kitty/tools/utils/humanize"
+	"github.com/kovidgoyal/kitty/tools/cli"
+	"github.com/kovidgoyal/kitty/tools/tui"
+	"github.com/kovidgoyal/kitty/tools/tui/loop"
+	"github.com/kovidgoyal/kitty/tools/utils"
+	"github.com/kovidgoyal/kitty/tools/utils/humanize"
 )
 
 var _ = fmt.Print
@@ -105,7 +106,7 @@ func edit_loop(data_to_send string, kill_if_signaled bool, on_data OnDataCallbac
 	lp.OnInitialize = func() (string, error) {
 		pos, chunk_num := 0, 0
 		for {
-			limit := utils.Min(pos+2048, len(data_to_send))
+			limit := min(pos+2048, len(data_to_send))
 			if limit <= pos {
 				break
 			}
@@ -212,7 +213,7 @@ func edit_in_kitty(path string, opts *Options) (err error) {
 	add_encoded("file_data", utils.UnsafeBytesToString(file_data))
 	fmt.Println("Waiting for editing to be completed, press Esc to abort...")
 	write_data := func(data_type string, rdata []byte) (err error) {
-		err = utils.AtomicWriteFile(path, rdata, fs.FileMode(s.Mode).Perm())
+		err = utils.AtomicWriteFile(path, bytes.NewReader(rdata), fs.FileMode(s.Mode).Perm())
 		if err != nil {
 			err = fmt.Errorf("Failed to write data to %s with error: %w", path, err)
 		}
