@@ -5,13 +5,13 @@ package main
 import (
 	"os"
 
-	"kitty/kittens/ssh"
-	"kitty/tools/cli"
-	"kitty/tools/cmd/completion"
-	"kitty/tools/cmd/tool"
+	"github.com/kovidgoyal/kitty/kittens/ssh"
+	"github.com/kovidgoyal/kitty/tools/cli"
+	"github.com/kovidgoyal/kitty/tools/cmd/completion"
+	"github.com/kovidgoyal/kitty/tools/cmd/tool"
 )
 
-func main() {
+func KittenMain(args ...string) {
 	krm := os.Getenv("KITTY_KITTEN_RUN_MODULE")
 	os.Unsetenv("KITTY_KITTEN_RUN_MODULE")
 	switch krm {
@@ -20,7 +20,8 @@ func main() {
 		return
 	}
 	root := cli.NewRootCommand()
-	root.ShortDescription = "Fast, statically compiled implementations for various kittens (command line tools for use with kitty)"
+	root.ShortDescription = "Fast, statically compiled implementations of various kittens (command line tools for use with kitty)"
+	root.HelpText = "kitten serves as a launcher for running individual kittens. Each kitten can be run as :code:`kitten command`. The list of available kittens is given below."
 	root.Usage = "command [command options] [command args]"
 	root.Run = func(cmd *cli.Command, args []string) (int, error) {
 		cmd.ShowHelp()
@@ -30,5 +31,9 @@ func main() {
 	tool.KittyToolEntryPoints(root)
 	completion.EntryPoint(root)
 
-	root.Exec()
+	root.Exec(args...)
+}
+
+func main() {
+	KittenMain()
 }
