@@ -4,6 +4,7 @@ package loop
 
 import (
 	"encoding/base64"
+	"encoding/hex"
 	"fmt"
 	"os"
 	"runtime"
@@ -12,10 +13,10 @@ import (
 
 	"golang.org/x/sys/unix"
 
-	"kitty/tools/tty"
-	"kitty/tools/utils"
-	"kitty/tools/utils/style"
-	"kitty/tools/wcswidth"
+	"github.com/kovidgoyal/kitty/tools/tty"
+	"github.com/kovidgoyal/kitty/tools/utils"
+	"github.com/kovidgoyal/kitty/tools/utils/style"
+	"github.com/kovidgoyal/kitty/tools/wcswidth"
 )
 
 type ScreenSize struct {
@@ -40,6 +41,7 @@ type Loop struct {
 	controlling_term                       *tty.Term
 	terminal_options                       TerminalStateOptions
 	screen_size                            ScreenSize
+	seen_inband_resize                     bool
 	escape_code_parser                     wcswidth.EscapeCodeParser
 	keep_going                             bool
 	death_signal                           unix.Signal
@@ -55,6 +57,10 @@ type Loop struct {
 	style_ctx                              style.Context
 	atomic_update_active                   bool
 	pointer_shapes                         []PointerShape
+	waiting_for_capabilities_response      bool
+
+	// Queried capabilities from terminal
+	TerminalCapabilities TerminalCapabilities
 
 	// Suspend the loop restoring terminal state, and run the provided function. When it returns terminal state is
 	// put back to what it was before suspending unless the function returns an error or an error occurs saving/restoring state.
@@ -89,6 +95,9 @@ type Loop struct {
 	// Called when a response to an rc command is received
 	OnRCResponse func(data []byte) error
 
+	// Called when a response to a query command is received
+	OnQueryResponse func(key, val string, valid bool) error
+
 	// Called when any input from tty is received
 	OnReceivedData func(data []byte) error
 
@@ -106,6 +115,15 @@ type Loop struct {
 
 	// Called on SIGTERM return true if you wish to handle it yourself
 	OnSIGTERM func() (bool, error)
+
+	// Called when capabilities response is received
+	OnCapabilitiesReceived func(TerminalCapabilities) error
+
+	// Called when the terminal's color scheme changes
+	OnColorSchemeChange func(ColorPreference) error
+
+	// Called on focus in/out events
+	OnFocusChange func(bool) error
 }
 
 func New(options ...func(self *Loop)) (*Loop, error) {
@@ -146,6 +164,15 @@ func OnlyDisambiguateKeys(self *Loop) {
 	self.terminal_options.kitty_keyboard_mode = DISAMBIGUATE_KEYS
 }
 
+func (self *Loop) NoKeyboardStateChange() *Loop {
+	self.terminal_options.kitty_keyboard_mode = NO_KEYBOARD_STATE_CHANGE
+	return self
+}
+
+func NoKeyboardStateChange(self *Loop) {
+	self.terminal_options.kitty_keyboard_mode = NO_KEYBOARD_STATE_CHANGE
+}
+
 func (self *Loop) FullKeyboardProtocol() *Loop {
 	self.terminal_options.kitty_keyboard_mode = FULL_KEYBOARD_PROTOCOL
 	return self
@@ -182,6 +209,19 @@ func NoRestoreColors(self *Loop) {
 	self.terminal_options.restore_colors = false
 }
 
+func (self *Loop) ColorSchemeChangeNotifications() *Loop {
+	self.terminal_options.color_scheme_change_notification = true
+	return self
+}
+
+func ColorSchemeChangeNotifications(self *Loop) {
+	self.terminal_options.color_scheme_change_notification = true
+}
+
+func NoInBandResizeNotifications(self *Loop) {
+	self.terminal_options.in_band_resize_notification = false
+}
+
 func (self *Loop) DeathSignalName() string {
 	if self.death_signal != SIGNULL {
 		return self.death_signal.String()
@@ -208,6 +248,27 @@ func (self *Loop) Println(args ...any) {
 	self.QueueWriteString("\r")
 }
 
+func (self *Loop) style_region(style string, start_x, start_y, end_x, end_y int) string {
+	sgr := self.SprintStyled(style, "|")
+	if len(sgr) > 2 {
+		sgr = sgr[2:strings.IndexByte(sgr, 'm')]
+		return fmt.Sprintf("\x1b[%d;%d;%d;%d;%s$r", start_y+1, start_x+1, end_y+1, end_x+1, sgr)
+	}
+	return ""
+}
+
+// Apply the specified style to the specified region of the screen (0-based
+// indexing). The region is all cells from the start cell to the end cell. See
+// StyleRectangle to apply style to a rectangular area.
+func (self *Loop) StyleRegion(style string, start_x, start_y, end_x, end_y int) IdType {
+	return self.QueueWriteString(self.style_region(style, start_x, start_y, end_x, end_y))
+}
+
+// Apply the specified style to the specified rectangle of the screen (0-based indexing).
+func (self *Loop) StyleRectangle(style string, start_x, start_y, end_x, end_y int) IdType {
+	return self.QueueWriteString("\x1b[2*x" + self.style_region(style, start_x, start_y, end_x, end_y) + "\x1b[*x")
+}
+
 func (self *Loop) SprintStyled(style string, args ...any) string {
 	f := self.style_cache[style]
 	if f == nil {
@@ -239,10 +300,7 @@ func (self *Loop) DebugPrintln(args ...any) {
 		const limit = 2048
 		msg := fmt.Sprintln(args...)
 		for i := 0; i < len(msg); i += limit {
-			end := i + limit
-			if end > len(msg) {
-				end = len(msg)
-			}
+			end := min(i+limit, len(msg))
 			self.QueueWriteString("\x1bP@kitty-print|")
 			self.QueueWriteString(base64.StdEncoding.EncodeToString([]byte(msg[i:end])))
 			self.QueueWriteString("\x1b\\")
@@ -253,11 +311,12 @@ func (self *Loop) DebugPrintln(args ...any) {
 func (self *Loop) Run() (err error) {
 	defer func() {
 		if r := recover(); r != nil {
-			pcs := make([]uintptr, 256)
+			pcs := make([]uintptr, 512)
 			n := runtime.Callers(2, pcs)
+			lines := []string{}
 			frames := runtime.CallersFrames(pcs[:n])
 			err = fmt.Errorf("Panicked: %s", r)
-			fmt.Fprintf(os.Stderr, "\r\nPanicked with error: %s\r\nStacktrace (most recent call first):\r\n", r)
+			lines = append(lines, fmt.Sprintf("\r\nPanicked with error: %s\r\nStacktrace (most recent call first):\r\n", r))
 			found_first_frame := false
 			for frame, more := frames.Next(); more; frame, more = frames.Next() {
 				if !found_first_frame {
@@ -266,8 +325,11 @@ func (self *Loop) Run() (err error) {
 					}
 					found_first_frame = true
 				}
-				fmt.Fprintf(os.Stderr, "%s\r\n\t%s:%d\r\n", frame.Function, frame.File, frame.Line)
+				lines = append(lines, fmt.Sprintf("%s\r\n\t%s:%d\r\n", frame.Function, frame.File, frame.Line))
 			}
+			text := strings.Join(lines, "")
+			os.Stderr.WriteString(text)
+			tty.DebugPrintln(strings.TrimSpace(text))
 			if self.terminal_options.Alternate_screen {
 				term, err := tty.OpenControllingTerm(tty.SetRaw)
 				if err == nil {
@@ -417,6 +479,10 @@ func (self *Loop) ClearScreen() {
 	self.QueueWriteString("\x1b[H\x1b[2J")
 }
 
+func (self *Loop) ClearScreenButNotGraphics() {
+	self.QueueWriteString("\x1b[H\x1b[J")
+}
+
 func (self *Loop) SendOverlayReady() {
 	self.QueueWriteString("\x1bP@kitty-overlay-ready|\x1b\\")
 }
@@ -454,6 +520,17 @@ func (self *Loop) CopyTextToClipboard(text string) {
 	self.copy_text_to(text, "c")
 }
 
+func (self *Loop) QueryTerminal(fields ...string) IdType {
+	if len(fields) == 0 {
+		return 0
+	}
+	q := make([]string, len(fields))
+	for i, x := range fields {
+		q[i] = hex.EncodeToString(utils.UnsafeStringToBytes("kitty-query-" + x))
+	}
+	return self.QueueWriteString(fmt.Sprintf("\x1bP+q%s\a", strings.Join(q, ";")))
+}
+
 func (self *Loop) PushPointerShape(s PointerShape) {
 	self.pointer_shapes = append(self.pointer_shapes, s)
 	self.QueueWriteString("\x1b]22;" + s.String() + "\x1b\\")
@@ -466,6 +543,8 @@ func (self *Loop) PopPointerShape() {
 	}
 }
 
+// Remove all pointer shapes from the shape stack resetting to default pointer
+// shape. This is called automatically on loop termination.
 func (self *Loop) ClearPointerShapes() (ans []PointerShape) {
 	ans = self.pointer_shapes
 	for i := len(self.pointer_shapes) - 1; i >= 0; i-- {
@@ -482,3 +561,15 @@ func (self *Loop) CurrentPointerShape() (ans PointerShape, has_shape bool) {
 	}
 	return
 }
+
+// Query the terminal for various capabilities, the OnCapabilitiesReceived
+// callback will be called once the query response is received. This
+// function should be called as early as possible ideally in OnInitialize.
+func (self *Loop) QueryCapabilities() {
+	if !self.waiting_for_capabilities_response {
+		self.waiting_for_capabilities_response = true
+		self.StartAtomicUpdate()
+		self.QueueWriteString("\x1b[?u\x1b[?996n\x1b[c")
+		self.EndAtomicUpdate()
+	}
+}
